

/* ***********************************************************************
                Naval Air Warfare Center / Aircraft Division              
                        Patuxent Naval Air Station                         
 ========================================================================

   File:           platform.c                                         
                                                                        
   Classification: UNCLASSIFIED                                   
                                                                        
   Description:    This file contains the functions, methods, and global
      variables needed to extract and manipulate platforms defined in the TDB
			
   Written by:     Alex Harper
*********************************************************************** */


#include "parse_tdb.h"


/* ***********************************************************
  This method extracts platform data from the TDB

   FILE *infile,                       // input file
   char *token,                        // last processed token
   int *errorflag                      // error flag
   dbl_llist_manager *clist,           // list of extracted capabilities
   dbl_llist_manager *susceptibility   // list of extracted susceptibilities
*********************************************************** */
void platform_type::extract_tdbdat(FILE *infile, char *token, int *errorflag, dbl_llist_manager *clist, dbl_llist_manager *susceptibility, dbl_llist_manager *semcode_table) {
      
   radar_type *ptr;
   susceptibility_type *atr;
   susceptibility_link *link;
                  
   *errorflag = get_sweg_token(infile, token);	// platform id

   *errorflag = get_sweg_token(infile, token);	// platform name
   name.stringcpy(token);

   if (semcode_table->head)
      semcode = ((name2sem *)semcode_table->head)->find(token);
   
   *errorflag = get_sweg_token(infile, token);

   while (*errorflag) {
      if (!strcmp(token, TOKEN_SNR_XMTR)) {
         xmit.insert(ptr = new radar_type, NULL);
         ptr->extract_tdbdat(infile, token, errorflag, clist, RADAR_TRANSMITTER, semcode_table);
         continue;
      }

      if (!strcmp(token, TOKEN_SNR_RCVR)) {
         rec.insert(ptr = new radar_type, NULL);
         ptr->extract_tdbdat(infile, token, errorflag, clist, RADAR_RECEIVER, semcode_table);
         continue;
      }

      if (!strcmp(token, TOKEN_SUSCEPTIBILITY)) {
         *errorflag = get_sweg_token(infile, token);

         atr = susceptibility->head ? ((susceptibility_type *)susceptibility->head)->find_table(token) : NULL;

         if (!atr) {
            susceptibility->insert(atr = new susceptibility_type, NULL);
            atr->name.stringcpy(token);
         }
	 
         em_reflect.insert(link = new susceptibility_link, NULL);
         link->list = atr;
	 
         *errorflag = get_sweg_token(infile, token);
         continue;
      }
      
      if (!strcmp(token, TOKEN_LINKAGES))
         break;
      if (!strcmp(token, TOKEN_PLATFORM))
         break;
      if (!strcmp(token, TOKEN_PLAYER_STRUCTURE))
         break;

      *errorflag = get_sweg_token(infile, token);
   }

}


/* ***********************************************************
  This method searches for a specific radar using the LINKAGE
  id specified in the TDB

  int x // linkage id of radar to find
*********************************************************** */
radar_type *platform_type::find_radar(int x) {

   radar_type *radar;

   if (xmit.head) {
      radar = ((radar_type *)xmit.head)->find_radar(x);
      if (radar)
         return radar;
   }

   if (rec.head) {
      radar = ((radar_type *)rec.head)->find_radar(x);
      if (radar)
         return radar;
   }

   if (next)
      return ((platform_type *)next)->find_radar(x);
      	    
   return NULL;
}


/* ***********************************************************
  This method dumps platform data to stdout

   name2sem *semcode_table,             // semantic code/name correlation table
   dbl_llist_manager *susceptibility,   // list of target susceptibilities
   char *prefix                         // prefix of tabs to print
*********************************************************** */
void platform_type::dump_data(dbl_llist_manager *susceptibility, char *prefix) {

   char buffer[256];
   radar_type *ptr;
   susceptibility_link *link;

   printf("%s%s\t ", prefix, name.string);

   if (semcode)
      printf("%d\n", semcode->semcode);
   else
      printf("Bad Semcode\n");
   
   sprintf(buffer, "%s      ", prefix);

   for (link=(susceptibility_link *)em_reflect.head; link; link=(susceptibility_link *)link->next) {
      printf("%s   Susceptibility:\t %s\n", prefix, link->list->name.string);

      if (susceptibility->head && ((susceptibility_type *)susceptibility->head)->find_table(link->list->name.string)) {
         if (link->list->rlist)
            link->list->rlist->dump_data(buffer);
         else
            printf("%s NO REFLECTIVE TABLE!!!\n", buffer);
         if (link->list->elist)
            link->list->elist->dump_data(buffer);
         else
            printf("%s NO EMITTIVE TABLE!!!\n", buffer);
      }

      else
         printf("*NOT FOUND*\n");
   }
      
   sprintf(buffer, "%s   ", prefix);
  
   for (ptr=(radar_type *)xmit.head; ptr; ptr=(radar_type *)ptr->next)
      ptr->dump_data(buffer);
   for (ptr=(radar_type *)rec.head; ptr; ptr=(radar_type *)ptr->next)
      ptr->dump_data(buffer);
}


/* ***********************************************************
  This method removes useless nodes after parsing the TDB
*********************************************************** */
void platform_type::data_cleanup() {

   radar_type *ptr, *qtr;
   susceptibility_link *etr, *ftr;
      
   // cleanup receivers
   ptr = (radar_type *)rec.head;
   
   while (ptr) {
      qtr = ptr;
      ptr = (radar_type *)ptr->next;

      qtr->data_cleanup();

      if (!qtr->query_justify()) {
         rec.remove(qtr);
         delete qtr;
      }
	 
   }

   // cleanup xmitters
   ptr = (radar_type *)xmit.head;
   
   while (ptr) {
      qtr = ptr;
      ptr = (radar_type *)ptr->next;

      qtr->data_cleanup();

      if (!qtr->query_justify()) {
         xmit.remove(qtr);
         delete qtr;
      }

   }

   // cleanup em_reflect links
   etr = (susceptibility_link *)em_reflect.head;
   
   while (etr) {
      ftr = etr;
      etr = (susceptibility_link *)etr->next;

      if (!ftr->list->rlist || !ftr->list->elist) {
         em_reflect.remove(ftr);
         delete ftr;
      }

   }

   
}


/* ***********************************************************
  This method determines if there is enough data extracted from the
  tdb to perform sensor calculations
*********************************************************** */
int platform_type::query_justify() {

   return (xmit.head || rec.head || em_reflect.head);
}


